@@ -1,5 +1,5 @@
 from re import split, sub
-from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple, Iterable
+from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple, Iterable, Mapping
 from sys import argv
 from os import getenv, name as os_name, environ as os_environ
 if os_name == 'nt':
@@ -8,7 +8,7 @@ from io import StringIO, BytesIO
 from collections import defaultdict
 from pathlib import Path
 
-__version__ = '2.1.10'
+__version__ = '2.1.11'
 GLOBALS_KEY = '_globals'
 
 if os_name == 'nt':
@@ -70,7 +70,7 @@ class DictConfig(dict):
     line arguments and allows for access using compound keys ('key.key') and global variable substitution
 
     :param args: arguments to be passed to the dict constructor
-    :param no_globals bool: if not set, the value of the GLOBALS_KEY item will be take to be a dict of globals
+    :param no_globals bool: if not set, the value of the GLOBALS_KEY item will be taken to be a dict of globals
         replacement values and this dict will be hidden from the DictConfig content
     :param no_key_error bool: if set, the DictConfig will not throw exceptions for non-existent keys (but return None)
     :param skip_lists bool: (deprecated 2.1.2, use skip_iterables) if True, casting should not recurse into lists
@@ -96,8 +96,8 @@ class DictConfig(dict):
         'request_header': 'rh', 'header': 'rh'
     }
 
-    def __init__(self, *args, no_globals: bool = False, no_key_error: bool = False, skip_lists: bool = False,
-                 no_compound_keys: bool = False, skip_iterables: bool = False):
+    def __init__(self, *args, no_globals: Union[bool, dict] = False, no_key_error: bool = False,
+                 skip_lists: bool = False, no_compound_keys: bool = False, skip_iterables: bool = False):
         """
         Constructor method
         """
@@ -108,8 +108,12 @@ class DictConfig(dict):
             self.globals = None
         elif no_globals:
             if isinstance(no_globals, dict):
-                # any dict type will be cast to type of self, with default settings
-                self.globals = self.__class__(no_globals)
+                if isinstance(no_globals, DictConfig):
+                    # copy the reference, if no_globals is already a Config
+                    self.globals = no_globals
+                else:
+                    # any other dict type will be cast (and thus copied) to type of self, with default settings
+                    self.globals = self.__class__(no_globals)
             else:
                 self.globals = None
         else:
@@ -312,6 +316,29 @@ class DictConfig(dict):
                     raise KeyError(f'Multi-part key, but `{keys[0]}` is not a dictionary or Config.')
             return self._subst_globals(super(DictConfig, self).__getitem__(keys[0]))
 
+    def get(self, key: str, default: Any = None) -> Any:
+        """
+        Override of get() that takes globals and compound keys into account
+        :param key: dict key of item to get
+        :param default: default to return if key is not found
+        :return: self[key]
+        """
+        return default if (result := self.__getitem__(key)) is None else result
+
+    def pop(self, key: str, default: Any = None) -> Any:
+        """
+        Override of pop() that takes globals and compound keys into account
+        :param key: dict key of item to pop
+        :param default: default to return if key is not found
+        :return: self[key] before it was removed
+        """
+        result = self.__getitem__(key)
+        if result is None:
+            return default
+        else:
+            del self[key]
+            return result
+
     def __setitem__(self, key: str, value: Any):
         """
         Set the item with (compound) key in self
@@ -327,6 +354,10 @@ class DictConfig(dict):
 
         keys = self._split_key(key)
 
+        if isinstance(value, dict):
+            if not isinstance(value, DictConfig):
+                value = self.__class__(value, no_globals=self.globals)
+
         if len(keys) == 0:
             raise KeyError(f'Invalid key value {key}.')
         elif len(keys) == 1:
@@ -336,11 +367,41 @@ class DictConfig(dict):
                 target = self[keys[:-1]]
                 target[keys[-1]] = value
             except KeyError:
-                self[keys[:-1]] = self.__class__({keys[-1]: value})
+                self[keys[:-1]] = self.__class__({keys[-1]: value}, no_globals=self.globals)
                 try:
                     self[keys[:-1]].shadow_attrs = self._shadow_attrs
                 except AttributeError:
                     pass
+        # once assignment was successful, update globals
+        if isinstance(value, DictConfig):
+            # when being loaded through pickle, it may not
+            if hasattr(self, 'globals'):
+                if hasattr(self, 'globals') and isinstance(value.globals, dict):
+                    new_globals = value.globals.copy()
+                    new_globals.update(self.globals)
+                    # update the main globals to match
+                    self.globals.update(new_globals)
+                    # the assigned globals to the main globals
+                    value.globals = self.globals
+                else:
+                    value.globals = self.globals
+
+    def __delitem__(self, key: str):
+        try:
+            if self.no_compound_keys:
+                return super(DictConfig, self).__delitem__(key)
+        except AttributeError:
+            pass
+
+        keys = self._split_key(key)
+
+        if len(keys) == 0:
+            raise KeyError(f'Invalid key value {key}.')
+        elif len(keys) == 1:
+            super(DictConfig, self).__delitem__(keys[0])
+        else:
+            target = self[keys[:-1]]
+            del target[keys[-1]]
 
     def dict_copy(self, skip_lists: bool = False, with_globals: bool = True, skip_iterables: bool = False) -> dict:
         """
@@ -365,6 +426,34 @@ class DictConfig(dict):
         """
         return self.__class__(self.dict_copy())
 
+    def update(self, other: Mapping = None, **kwargs) -> 'DictConfig':
+        if other is not None:
+            if hasattr(other, 'globals'):
+                if self.globals is not None:
+                    self.globals.update(other.globals)
+                else:
+                    if other.globals is not None:
+                        self.globals = other.globals.copy()
+            for k, v in other.items():
+                if k in self and isinstance(self[k], DictConfig):
+                    self[k].update(v)
+                else:
+                    self[k] = v
+        if kwargs:
+            self.update(kwargs)
+        return self
+
+    def resolve_imports(self, prefix='import@'):
+        if not prefix:
+            raise SyntaxError('Empty prefix for resolve_imports not allowed.')
+        for k, v in self.items():
+            if isinstance(v, str) and v.startswith(prefix):
+                # not referencing v, but self[k] to trigger global replacements
+                self[k] = self.__class__.load(self[k][len(prefix):], no_arguments=True)
+            elif isinstance(v, DictConfig):
+                v.resolve_imports(prefix)
+        return self
+
     @classmethod
     def _xml2cfg(cls, root, **kwargs):
         """
@@ -927,3 +1016,11 @@ class Config(DictConfig):
             super(DictConfig, self).__setattr__(attr, value)
         else:
             self[attr] = value
+
+    def __delattr__(self, attr):
+        if (attr in dir(self) or attr not in self) and (not hasattr(self, '_shadow_attrs') or not self._shadow_attrs):
+            super(DictConfig, self).__delattr__(attr)
+        elif attr in self:
+            del self[attr]
+        else:
+            raise AttributeError(f'No attribute or key {attr} for {self.__class__}')
